@@ -1,45 +1,58 @@
-from typing import Optional, Union, List, Tuple, Dict, Literal
+from typing import Optional, Union, List, Dict, Literal
 from collections import OrderedDict
 import math
 
 import torch
 from torch import nn
+try:
+    from typing_extensions import Final
+except:
+    from torch.jit import Final
 
-from comvex.utils import PathDropout
+from comvex.utils import PathDropout, XXXConvXdBase
 from comvex.utils.helpers import name_with_msg, get_attr_if_exists, config_pop_argument
 from .config import EfficientNetConfig
 
 
 class EfficientNetBase(nn.Module):
-    __constants__ = [
-        "num_layers",
-        "channels",
-        "kernel_sizes",
-        "strides",
-        "expand_scales",
-        "se_scales",
-        "resolution",
-        "return_feature_maps",
-    ]
+    num_layers: Final[List[int]]
+    channels: Final[List[int]]
+    kernel_sizes: Final[List[int]]
+    strides: Final[List[int]]
+    expand_scales: Final[List[Optional[int]]]
+    se_scales: Final[List[Optional[float]]]
+    resolution: Final[int]
+    return_feature_maps: Final[bool]
 
     def __init__(
         self,
         depth_scale: float,
         width_scale: float,
         resolution: int,
-        se_scale: float = 0.25,
         up_sampling_mode: Optional[str] = None,  # Check out: https://pytorch.org/docs/stable/generated/torch.nn.Upsample.html?highlight=up%20sample#torch.nn.Upsample
         return_feature_maps: bool = False,
+        num_layers: Optional[List[int]] = None,
+        channels: Optional[List[int]] = None,
+        kernel_sizes: Optional[List[int]] = None,
+        strides: Optional[List[int]] = None,
+        expand_scales: Optional[List[Optional[int]]] = None,
+        se_scales: Optional[List[Optional[int]]] = None,
+        se_scale: Optional[float] = 0.25,
     ) -> None:
         super().__init__()
 
         # Table 1. from the official paper (all stages)
-        self.num_layers = self.scale_and_round_layers([1, 1, 2, 2, 3, 3, 4, 1, 1], depth_scale)
-        self.channels = self.scale_and_round_channels([32, 16, 24, 40, 80, 112, 192, 320, 1280], width_scale)
-        self.kernel_sizes = [3, 3, 3, 5, 3, 5, 5, 3, 1]    
-        self.strides = [1, 2, 1, 2, 2, 2, 1, 2, 1]
-        self.expand_scales = [None, 1, 6, 6, 6, 6, 6, 6, None]
-        self.se_scales = [None, *((se_scale,)*7), None]
+        self.num_layers = self.scale_and_round_layers(num_layers if num_layers is not None else [1, 1, 2, 2, 3, 3, 4, 1, 1], depth_scale)
+        self.channels = self.scale_and_round_channels(channels if channels is not None else [32, 16, 24, 40, 80, 112, 192, 320, 1280], width_scale)
+        self.kernel_sizes = kernel_sizes if kernel_sizes is not None else [3, 3, 3, 5, 3, 5, 5, 3, 1]    
+        self.strides = strides if strides is not None else [1, 2, 1, 2, 2, 2, 1, 2, 1]
+        self.expand_scales = expand_scales if expand_scales is not None else [None, 1, 6, 6, 6, 6, 6, 6, None]
+
+        assert (
+            se_scales is not None or se_scale is not None
+        ), name_with_msg("Either `se_scales` or `se_scale` should be specified")
+        
+        self.se_scales = se_scales if se_scales is not None else [None, *((se_scale,)*7), None]
 
         self.resolution = resolution
         # From: https://github.com/tensorflow/tpu/blob/3679ca6b979349dde6da7156be2528428b000c7c/models/official/efficientnet/preprocessing.py#L88
@@ -47,6 +60,7 @@ class EfficientNetBase(nn.Module):
         self.up_sampling_mode = up_sampling_mode
         self.return_feature_maps = return_feature_maps
 
+    @torch.jit.ignore
     def scale_and_round_layers(self, in_list: List[int], scale) -> List[int]:
         out = list(map(lambda x: int(math.ceil(x*scale)), in_list))
         out[0] = 1  # Stage 1 always has one layer
@@ -54,6 +68,7 @@ class EfficientNetBase(nn.Module):
         
         return out
 
+    @torch.jit.ignore
     def scale_and_round_channels(self, in_list: List[int], scale) -> List[int]:
         r"""
         Reference from: https://github.com/tensorflow/tpu/blob/3679ca6b979349dde6da7156be2528428b000c7c/models/official/efficientnet/efficientnet_model.py#L106
@@ -80,7 +95,7 @@ class EfficientNetBase(nn.Module):
         return new_filters
 
 
-class SeperateConvXd(nn.Module):
+class SeperateConvXd(XXXConvXdBase):
     r"""
     Reference from: MnasNet (https://arxiv.org/pdf/1807.11626.pdf)
     """
@@ -96,64 +111,82 @@ class SeperateConvXd(nn.Module):
         act_fnc_name: str = "ReLU",
         dimension: int = 2,
         **kwargs  # For the normalization layer
-    ):
-        super().__init__()
-
-        assert (
-            (0 < dimension) and (dimension < 4)
-        ), name_with_msg(self, "`dimension` must be larger than 0 and smaller than 4")
-
-        if dimension == 1:
-            conv = nn.Conv1d
-        elif dimension == 2:
-            conv = nn.Conv2d
-        else: 
-            conv = nn.Conv3d
+    ) -> None:
+        super().__init__(in_channel, out_channel, dimension=dimension)
 
         self.depth_wise_conv = nn.Sequential(
-            conv(
-                in_channel, 
-                in_channel*kernels_per_layer, 
+            self.conv(
+                self.in_channel, 
+                self.in_channel*kernels_per_layer, 
                 kernel_size, 
                 padding=padding, 
-                groups=in_channel
+                groups=self.in_channel
             ),
             get_attr_if_exists(nn, norm_layer_name)(
-                in_channel*kernels_per_layer,
+                self.in_channel*kernels_per_layer,
                 **kwargs
             ),
             get_attr_if_exists(nn, act_fnc_name)()
         )
         self.pixel_wise_conv = nn.Sequential(
-            conv(
-                in_channel*kernels_per_layer,
+            self.conv(
+                self.in_channel*kernels_per_layer,
                 out_channel,
                 kernel_size=1,
             ),
             get_attr_if_exists(nn, norm_layer_name)(
-                in_channel*kernels_per_layer,
+                self.in_channel*kernels_per_layer,
                 **kwargs
             )
         )
 
-    def forward(self, x):
+    def forward(self, x: torch.Tensor) -> torch.Tensor:
         x = self.depth_wise_conv(x)
         x = self.pixel_wise_conv(x)
 
         return x
 
 
-class MBConvXd(nn.Module):
+class SEConvXd(XXXConvXdBase):
+    r"""
+    Squeeze-and-Excitation Convolution 
+
+    From: Squeeze-and-Excitation Networks (https://arxiv.org/pdf/1709.01507.pdf)
+    """
+    def __init__(
+        self,
+        in_channel: int,
+        bottleneck_channel: int,
+        out_channel: Optional[int] = None,
+        se_act_fnc_name: Optional[str] = None,
+        dimension: int = 2,
+    ) -> None:
+        super().__init__(in_channel, out_channel, dimension=dimension)
+
+        self.layers = nn.Sequential(
+            nn.AdaptiveMaxPool2d((1, 1)),
+            self.conv(self.in_channel, bottleneck_channel, 1),
+            get_attr_if_exists(nn, se_act_fnc_name)(),
+            self.conv(bottleneck_channel, self.out_channel, 1),
+            nn.Sigmoid(),
+        )
+
+    def forward(self, x: torch.Tensor) -> torch.Tensor:
+        return x*self.layers(x)        
+
+
+class MBConvXd(XXXConvXdBase):
     r"""
     Reference from: 
         1. MobileNetV2 (https://arxiv.org/pdf/1801.04381.pdf)
         2. MnasNet (https://arxiv.org/pdf/1807.11626.pdf)
-        3. Squeeze-and-Excitation Networks (https://arxiv.org/pdf/1709.01507.pdf)
-        4. Official Implementation (https://github.com/tensorflow/tpu/blob/master/models/official/efficientnet/efficientnet_model.py)
+        3. Official Implementation (https://github.com/tensorflow/tpu/blob/master/models/official/efficientnet/efficientnet_model.py)
     
     Note: `Swish` is called `SiLU` in PyTorch
     """
 
+    skip: Final[bool]
+
     def __init__(
         self, 
         in_channel: int,
@@ -172,20 +205,7 @@ class MBConvXd(nn.Module):
         expansion_head_type: Literal["pixel_depth", "fused"] = "pixel_depth",
         **kwargs  # For example: `eps` and `elementwise_affine` for `nn.LayerNorm`
     ):
-        super().__init__()
-
-        out_channel = out_channel if out_channel is not None else in_channel
-
-        assert (
-            (0 < dimension) and (dimension < 4)
-        ), name_with_msg(self, "`dimension` must be larger than 0 and smaller than 4")
-
-        if dimension == 1:
-            conv = nn.Conv1d
-        elif dimension == 2:
-            conv = nn.Conv2d
-        else: 
-            conv = nn.Conv3d
+        super().__init__(in_channel, out_channel, dimension=dimension)
 
         assert (
             expand_channel is not None or expand_scale is not None
@@ -202,8 +222,8 @@ class MBConvXd(nn.Module):
         # Expansion Head
         if expansion_head_type == "pixel_depth":
             pixel_wise_conv_0 = nn.Sequential(
-                conv(
-                    in_channel,
+                self.conv(
+                    self.in_channel,
                     expand_channel,
                     kernel_size=1,
                     bias=False
@@ -216,7 +236,7 @@ class MBConvXd(nn.Module):
             )
 
             depth_wise_conv = nn.Sequential(
-                conv(
+                self.conv(
                     expand_channel, 
                     expand_channel, 
                     kernel_size, 
@@ -239,7 +259,7 @@ class MBConvXd(nn.Module):
         else:
             self.expansion_head = nn.Sequential(
                 nn.Conv2d(
-                    in_channel,
+                    self.in_channel,
                     expand_channel,
                     kernel_size,
                     stride=stride,
@@ -258,24 +278,22 @@ class MBConvXd(nn.Module):
         if se_scale is not None:
             bottleneck_channel = int(expand_channel*se_scale)
 
-            self.se_block = nn.Sequential(
-                nn.AdaptiveMaxPool2d((1, 1)),
-                nn.Conv2d(expand_channel, bottleneck_channel, 1),
-                get_attr_if_exists(nn, se_act_fnc_name)(),
-                nn.Conv2d(bottleneck_channel, expand_channel, 1),
-                nn.Sigmoid(),
+            self.se_block = SEConvXd(
+                expand_channel,
+                bottleneck_channel,
+                se_act_fnc_name=se_act_fnc_name,
             )
 
         #
         self.pixel_wise_conv_1 = nn.Sequential(
-            conv(
+            self.conv(
                 expand_channel,
-                out_channel,
+                self.out_channel,
                 kernel_size=1,
                 bias=False,
             ),
             get_attr_if_exists(nn, norm_layer_name)(
-                out_channel,
+                self.out_channel,
                 **kwargs
             )
         )
@@ -283,16 +301,16 @@ class MBConvXd(nn.Module):
         # From: https://github.com/tensorflow/tpu/blob/3679ca6b979349dde6da7156be2528428b000c7c/models/official/efficientnet/utils.py#L276
         # It's a batch-wise dropout
         self.path_dropout = PathDropout(path_dropout)
-        self.skip = True if in_channel == out_channel and stride == 1 else False
+        self.skip = True if self.in_channel == self.out_channel and stride == 1 else False
 
-    def forward(self, x):
+    def forward(self, x: torch.Tensor) -> torch.Tensor:
 
         # expansion head
-        res = self.expansion_head(x)
+        res: torch.Tensor = self.expansion_head(x)
 
         # SE block
         if self.se_block is not None:
-            res = res*self.se_block(res)
+            res = self.se_block(res)
 
         # Second pixel-wise conv
         res = self.pixel_wise_conv_1(res)
@@ -303,6 +321,7 @@ class MBConvXd(nn.Module):
         return x + res if self.skip else res
 
     @classmethod
+
     def MBConv2d6k3(cls, in_channel: int, out_channel: int, **kwargs) -> "MBConvXd":
         r"""
         The MBConv6 (k3x3) from MnasNet (https://arxiv.org/pdf/1807.11626.pdf)
@@ -371,7 +390,7 @@ class EfficientNetBackbone(EfficientNetBase):
         )
 
     def forward(self, x: torch.Tensor) -> Union[
-        Tuple[torch.Tensor, Dict[str, torch.Tensor]],
+        Dict[str, torch.Tensor],
         torch.Tensor
     ]:
         # These `if` statements should be removed after scripting, so don't worry
@@ -383,7 +402,7 @@ class EfficientNetBackbone(EfficientNetBase):
             )
 
         if self.return_feature_maps:
-            feature_maps = {}
+            feature_maps: Dict[str, torch.Tensor] = {}
 
         x = self.stage_1(x)
         x = self.stage_2(x)
@@ -409,8 +428,9 @@ class EfficientNetBackbone(EfficientNetBase):
         if self.return_feature_maps:
             feature_maps['reduction_5'] = x
             
-        return (x, feature_maps) if self.return_feature_maps else x
+        return feature_maps if self.return_feature_maps else x
 
+    @torch.jit.ignore
     def _build_stage(self, stage_idx: str, **kwargs) -> nn.Module:
         access_idx = int(stage_idx) - 1  # Since the naming of stages is not 0-based
         num_stages = len(self.num_layers)
@@ -460,6 +480,7 @@ class EfficientNetBackbone(EfficientNetBase):
                 )
             ]))
 
+    @torch.jit.ignore
     def num_parameters(self) -> int:
         return sum(params.numel() for _, params in self.named_parameters())
 
@@ -478,14 +499,16 @@ class EfficientNetWithLinearClassifier(EfficientNetBackbone):
         )
 
     def forward(self, x: torch.Tensor) -> Union[
-        Tuple[torch.Tensor, Dict[str, torch.Tensor]],
+        Dict[str, torch.Tensor],
         torch.Tensor
     ]:
-        b, *rest = x.shape
+        b = x.shape[0]
+
         if self.return_feature_maps:
-            x, feature_maps = super().forward(x)
+            feature_maps: Dict[str, torch.Tensor] = super().forward(x)
+            x: torch.Tensor = feature_maps['reduction_5']
         else:
-            x = super().forward(x)
+            x: torch.Tensor = super().forward(x)
 
         # (B, C, H, W) -> (B, C, 1, 1) -> (B, C), avoid einops here now for scripting, edit it when einops is scriptable
         x = self.pooler(x).view(b, -1)
@@ -493,5 +516,5 @@ class EfficientNetWithLinearClassifier(EfficientNetBackbone):
         x = self.ff_dropout(x)
         x = self.proj_head(x)
 
-        return (x, feature_maps) if self.return_feature_maps else x
+        return { "x": x, **feature_maps } if self.return_feature_maps else x
 
